;**********************************************************
;*  This program is for exericse 10, the program will get A/D
;*  result for LEDs shift time control on PORTC. 
;*  Please make sure that the DIP switch SW3 has all 
;*  switches in the ON position.
;**********************************************************
;
			list p=16F877a		
#include 	<p16f877a.inc>			; Include file locate at defult directory
;
#define		RA4	PORTA,4
;
SHIFT_VAL	equ	b'00000111'			; LED shift initial value
;
VAL_500US	equ	.250				; 0.5ms delay value
VAL_10MS	equ	.20					; 10mS delay value
;
count_us	equ	0x20				; Defined temp reg. for 1ms delay
count_10ms	equ	0x21           	 	; Defined delay reg.
count_200ms	equ	0x22
count_ms	equ	0x23
flag_temp	equ	0x24
;***************************************
;           Program start              *
;***************************************
			org 	0x00			; reset vector
			nop						; Reserve for MPLAB-ICD
			goto	initial
;
			org		0x04			;Interrupt vector
			retfie
initial: 				
			clrf	PCLATH
			clrf	INTCON			;Disable all Interrupt
;		
			banksel	TRISD			; Select to bank1
			clrf	TRISD			; Set PORTD for LED output
;
			movlw	b'00000111'		; Disable Voltage Comparator Module at PORTA
			movwf	CMCON
;
			banksel PORTD 			; Select to bank0
			clrf	PORTD			; Clear PORTC
;
init_ad 
			movlw	b'11000001'		; A/D osc source from internal RC
			movwf	ADCON0			; Select CH0, Turn on A/D
;		
			banksel	ADCON1			; Bank1 register
			movlw	b'00001110'		; Left justify, CH0 is for A/D input
			movwf	ADCON1
			banksel ADCON0			; Change back to Bank0
;
;************ Main *********************
;
start: 
			movlw	SHIFT_VAL		; Load display data
			movwf	PORTD
;
test_rb0	btfss	RA4				; Check RA4 (SW) status
			goto	led_right
;
led_left	rlf		PORTD,F			; SW is OFF, LEDs shift LEFTwf	
			swapf	STATUS,W		; Push STATUS
			movwf	flag_temp	
  			call 	a_d_convert 	; Call A/D convertor
			call	low_adjust		; Adjust A/D result for delay time
 			call	delay_200ms		; Call delay subroutine
			swapf	flag_temp,W		; Pop STATUS
			movwf	STATUS
			goto	test_rb0
;
led_right	rrf		PORTD,F			; SW is ON , LEDs shift RIGHT
			swapf	STATUS,W
			movwf	flag_temp
			call 	a_d_convert
			call	low_adjust		; Adjust A/D result for delay time
			call	delay_200ms		; Call delay subroutine
			swapf	flag_temp,W
			movwf	STATUS
			goto	test_rb0
;
;-------- A/D convert routine -----------
;
a_d_convert: 
			bsf		ADCON0,GO		; StartA/Dconvert
			call 	delay_500us
a_d_wait	nop
  			btfss	PIR1,ADIF		; Check A/D done?
			goto	a_d_wait
;
			bcf		PIR1,ADIF		; Clear flag of Convert comleted 
			movf	ADRESH,W		; Get 8 bit MSB from A/D result register
			return	
;
;-------- Adjust A/D result when = 0 ----
;
low_adjust:
			movwf	count_ms
			sublw	d'3'			; 3 - W , W <= 3 ; C = 1 
			btfss	STATUS,C		; Test A/D result (W) <= 3 ?
			return					; > 5 used original A/D result
			movlw	d'2'			; < 5 adjust base delay time to 2 
			movwf	count_ms
			return		
;
;--------- 200 md delay routine --------
;
delay_200ms:	
			movf	count_ms,W		 
			movwf	count_200ms
loop_20ms	call 	delay_10ms
			decfsz	count_200ms,F
			goto	loop_20ms
			return
;
;-------- 10 ms delay routine ----------
;
delay_10ms:	
			movlw	VAL_10MS		 
			movwf	count_10ms
loop_ms		call 	delay_500us
			decfsz	count_10ms,F
			goto	loop_ms
			return
;
;-------- 0.5 ms delay routine -----------
;
delay_500us:
			movlw	VAL_500US		 		
			movwf	count_us
dec_loop	nop
			nop
			nop
			nop
			nop
			decfsz	count_us,F
			goto 	dec_loop
			return
;	
			end
